;  Name: fsalbt
;  -----
;                     
;  Purpose :
;  --------
;     used for the computation of the the local Brunt-Vaisala frequency 
;	
;   Input : potential temperature tn, salinity sn
;   ------
;
;   References :
;   -----------
;	McDougall, T. J., J. Phys. Oceanogr., 17, 1950-1964, 1987.
;
;   Modifications :
;   --------------
;      Original  : 94-07 (G. Madec, M. Imbard)
;      Additions : 97-07 (G. Madec) introduction of statement functions
;      Idl version : 98-12 (M. Levy)
;----------------------------------------------------------------------
FUNCTION fsalbt, pft, pfs, pfh
;
; ratio alpha/beta
; ================
;
;  fsalbt: ratio of thermal over saline expension coefficients
;       pft :  potential temperature in degrees celcius
;       pfs :  salinity anomaly (s-35) in psu
;       pfh :  depth in meters

      z = $ 
       ( ( ( -0.255019e-07 * pft + 0.298357e-05 ) * pft $ 
                                 - 0.203814e-03 ) * pft $ 
                                 + 0.170907e-01 ) * pft $ 
                                 + 0.665157e-01 $ 
      +(-0.678662e-05 * pfs - 0.846960e-04 * pft + 0.378110e-02 ) * pfs $ 
      +  ( ( - 0.302285e-13 * pfh $ 
             - 0.251520e-11 * pfs $ 
             + 0.512857e-12 * pft * pft          ) * pfh $ 
                                  - 0.164759e-06   * pfs $ 
          +(   0.791325e-08 * pft - 0.933746e-06 ) * pft $ 
                                  + 0.380374e-04 ) * pfh 
 
return,  z 
end 
 
